/**************************************************************************
 *    WetPaint 1.2
 *    COPYRIGHT 1993 BY DAVID KOSKI AND GARRICK TOUBASSI
 *    ALL RIGHTS RESERVED
 **************************************************************************/

#import "ColorCorrectionFilter.h"

@implementation ColorCorrectionFilter

- filterWasLoaded:manager
{
    filterManager = manager;
    filterBundle = [NXBundle bundleForClass:[self class]];
    
    tolerance = 1;
    
    return self;
}

- filterWillUnload
{
    if (iconImage)
	[iconImage free];
    if (inspectorView)
	[[inspectorView window] free];
	
    return self;
}

- filterImage
{
    if (!iconImage) {
        char	buf[MAXPATHLEN + 1];

	if ([filterBundle getPath:buf forResource:"ColorCorrection"
		ofType:"tiff"])
	    iconImage = [[NXImage alloc] initFromFile:buf];
    }
    
    return iconImage;
}



- filterInspector
{
    if (!inspectorView) {
        char	buf[MAXPATHLEN + 1];

	if ([filterBundle getPath:buf forResource:"ColorCorrectionFilter" 
	    ofType:"nib"]) {
	    [NXApp loadNibFile:buf owner:self];
	}
    }
    
    return inspectorView;
}

- (unsigned int)whichCommonAttrs
{
    return ALL_ATTRS;
}


- toleranceChanged:sender
{
    if (sender == toleranceSlider) {
	tolerance = [sender doubleValue];
	[toleranceText setDoubleValue: tolerance * 100];
    } else {
    	tolerance = [sender doubleValue] / 100;
	[toleranceSlider setDoubleValue: tolerance];
    }
    return self;
}


//--------------------------------------------------------------------------
//	Image Processing Methods
//--------------------------------------------------------------------------

- (NXImage *)applyFilter:(NXImage *)theImage
{
    NXBitmapImageRep	*im;
    unsigned char	*data;
    unsigned char	src[4], dest[4];
    int			i, j, wrap = 0, delta[4], total;
    unsigned char	min[4], max[4];
    
    [self getChannels];
    
    // set up the function
    [self convertColor: [srcColor color] : colorSpace : src];
    [self convertColor: [destColor color] : colorSpace : dest];
    for (i = 0; i < 4; i ++)
	delta[i] = chan[i] ? dest[i] - src[i] : 0;
    // we have a special case for HSB mode where the hue range wraps
    // from 255 -> 0
    if (colorSpace == CS_HSB) {
	min[0] = chan[0] ? ((int)(src[0] - tolerance * 255)) & 0xff : 0;
	max[0] = chan[0] ? ((int)(src[0] + tolerance * 255)) & 0xff : 0;
	if (max[0] < min[0])
	    wrap = 1;
	else
	    wrap = 0;
	for (i = 1; i < 4; i ++) {
	    min[i] = chan[i] ? MAX(0, src[i] - tolerance * 255) : 255;
	    max[i] = chan[i] ? MIN(255, src[i] + tolerance * 255) : 0;
	}
    } else 
	for (i = 0; i < 4; i ++) {
	    min[i] = chan[i] ? MAX(0, src[i] - tolerance * 255) : 255;
	    max[i] = chan[i] ? MIN(255, src[i] + tolerance * 255) : 0;
	}

    // convert the data to a reasonable form
    im = [self toDepth: NX_TwentyFourBitRGBDepth :theImage];
    [self toColorSpace: im : colorSpace];
    data = planes[0];
    
    // process
    total = pixelsHigh * bytesPerRow;
    if (colorSpace == CS_HSB) {
	for (i = 0; i < total; i += 4) {
	    if ((!wrap && data[0] <= max[0] && data[0] >= min[0]) ||
	    	(wrap && (data[0] <= max[0] || data[0] >= min[0])))
		data[0] = (data[0] + delta[0]) & 0xff;
	    for (j = 1; j < 4; j ++)
		if (data[j] <= max[j] && data[j] >= min[j])
		    data[j] = MIN(255, MAX(0, data[j] + delta[j]));
	    data += 4;
	    if ((i & 0x3ff) == 0)
		[filterManager doneWith: 
			(double) i / (double) total];
	}
    } else 
	for (i = 0; i < total; i += 4) {
	    for (j = 0; j < 4; j ++)
		if (data[j] <= max[j] && data[j] >= min[j])
		    data[j] = MIN(255, MAX(0, data[j] + delta[j]));
	    data += 4;
	    if ((i & 0x3ff) == 0)
		[filterManager doneWith: 
			(double) i / (double) total];
	}
    
    // now ship it back    
    [self fromColorSpace: im : colorSpace];
    
    theImage = [self drawBitmap:im];
    [im free];
    return theImage;
}


@end
