/**************************************************************************
 *    WetPaint 1.2
 *    COPYRIGHT 1993 BY DAVID KOSKI AND GARRICK TOUBASSI
 *    ALL RIGHTS RESERVED
 **************************************************************************/

#import "SoftenFilter.h"
#import "../circle.h"

@implementation SoftenFilter

- filterWasLoaded:manager
{
    filterManager = manager;
    filterBundle = [NXBundle bundleForClass:[self class]];
    
    return self;
}

- filterWillUnload
{
    if (iconImage)
	[iconImage free];
    if (inspectorView)
	[[inspectorView window] free];
	
    return self;
}

- filterImage
{
    if (!iconImage) {
        char	buf[MAXPATHLEN + 1];

	if ([filterBundle getPath:buf forResource:"soften" ofType:"tiff"])
	    iconImage = [[NXImage alloc] initFromFile:buf];
    }
    
    return iconImage;
}



- filterInspector
{
    if (!inspectorView) {
        char	buf[MAXPATHLEN + 1];

	if ([filterBundle getPath:buf forResource:"SoftenFilter" 
	    ofType:"nib"]) {
	    [NXApp loadNibFile:buf owner:self];
	}
    }
    
    return inspectorView;
}

- widthChanged:sender;
{
    [widthText setIntValue: [widthSlider intValue]];

    return self;
}


- (unsigned int)whichCommonAttrs
{
    return NO_ATTRS;
}



//--------------------------------------------------------------------------
//	Image Processing Methods
//--------------------------------------------------------------------------

- (NXImage *)applyFilter:(NXImage *)theImage
{
    int		width;
    double	da;
    int		x;
    NXImage	*im;
    NXSize	size;
    NXRect	rect;
    NXPoint	zero = {0.0, 0.0};
    int		mode = [[modeMatrix selectedCell] tag];
    
    [theImage getSize: &size];
    
    im = [[[NXImage alloc] setSize: &size] setCacheDepthBounded: NO];
    [im useCacheWithDepth: NX_TwentyFourBitRGBDepth];
   
    width = [widthSlider intValue];
    width = MIN(width, MIN(size.width, size.height) / 2);
    
    da = 1.0 / width;
    
    [im lockFocus];
    NXSetColor(NX_COLORWHITE);
    if (mode == M_CIRC)
	PSsetalpha(0);
    NXSetRect(&rect, 0, 0, size.width, size.height);
    NXRectFill(&rect);
    
    for (x = 0; x < width; x++) {
	NXSetRect(&rect, x, x, size.width - (2 * x), size.height - (2 * x));
	PSsetalpha(da * x);
	switch (mode) {
	case M_CIRC:
	    PSFilledOval(NX_X(&rect), NX_Y(&rect), 
	    	NX_WIDTH(&rect), NX_HEIGHT(&rect));
	    break;
	case M_RECT:
	    NXFrameRect(&rect);
	    break;
	}
    }
    
    NXSetRect(&rect, x, x, size.width - (2 * x), size.height - (2 * x));
    PSsetalpha(1);
    switch (mode) {
    case M_CIRC:
	PSFilledOval(NX_X(&rect), NX_Y(&rect), 
	    NX_WIDTH(&rect), NX_HEIGHT(&rect));
	break;
    case M_RECT:
	break;
    }
    
    [theImage composite: NX_SIN toPoint: &zero];
    
    [im unlockFocus];
    
    return im;
}


@end
